@@ -1,4 +1,4 @@
-#!usr/env/bin python
+#!/usr/bin/env python
 
 
 """Script for automatically downloading a list of videos, speeding them up, and concatenating them.
@@ -12,20 +12,19 @@ Files (configurable):
 
 
 import re
-import os
 import sys
 import argparse
 import concurrent.futures
+from pathlib import Path
 
 import youtube_dl as youtube_yl  # youtube yownloader
 import ffmpeg
 
 
-__version__ = '0.7.0'
+__version__ = '0.9.1'
 
 
 # TODO: command-line argument support
-# TODO: - vod urls as a list of arguments
 # TODO: - option to turn off logging for youtube_dl and ffmpeg
 # TODO: handle vod list input through input() after running
 # TODO: option to disable multithreading or use a batch mode for large lists
@@ -38,25 +37,33 @@ YOUTUBE_DL_DEFAULT_OUTTMPL = '%(title)s-%(id)s.%(ext)s'
 
 # command line interface
 parser = argparse.ArgumentParser(description='Script for automatically downloading a list of videos, speeding them up, '
-                                             'and concatenating them.')
+                                             'and concatenating them.',
+                                 usage='%(prog)s [-h] [--version] [OPTIONS] [VIDEO_URLS ...]')
 parser.add_argument('--version', action='version', version=__version__)
 
-parser.add_argument('cmd_vods_list', nargs='*', default=[])
+parser.add_argument('video_urls', nargs='*', default=[])
 
+parser.add_argument('-v', '--verbose', action='store_true', dest='verbose',
+                    help='show output from youtube-dl and ffmpeg')
 parser.add_argument('-f', '--file', default='vods.txt', dest='vods_list_file_path',
                     help="path to file with list of video URL's, used if they are not given as arguments")
-parser.add_argument('-of', '--out-folder', '--output', default='downloads', dest='out_folder',
+parser.add_argument('-o', '--out-folder', '--output', default='downloads', dest='out_folder',
                     help="folder to download to, default: '%(default)s'")
 parser.add_argument('--overwrite', action='store_true', dest='clear_out_folder',
                     help='delete any and all files in the output folder before starting')
+parser.add_argument('--no-clear', action='store_true', dest='no_clear',
+                    help="Don't try to clear the out folder before starting. Allows resuming downloads, "
+                    'but may mess up concatenation.')
 parser.add_argument('-b', '--prefer-best-quality', action='store_true', dest='prefer_best_quality',
                     help='720p or lower will be used unless this option is selected, warning: may break concat')
 parser.add_argument('-s', '--speed', type=int, default=1000, dest='speed',
                     help='multiplier for speeding up the video for ffmpeg, default: %(default)s')
 parser.add_argument('-n', '--output-timelapse-name', default='_timelapse.mp4', dest='output_timelapse_filename',
                     help='name of final output, default: %(default)s, note:should include file extension, for ffmpeg')
-parser.add_argument('-k', '--keep-timelapse-parts', action='store_true', dest='keep_timelapse_parts',
+parser.add_argument('-kt', '--keep-timelapse-parts', action='store_true', dest='keep_timelapse_parts',
                     help='individual sped up videos will be deleted unless this option is selected')
+parser.add_argument('-ko', '--keep-original-parts', action='store_true', dest='keep_original_parts',
+                    help='individual full length videos will be deleted unless this option is selected')
 
 args = parser.parse_args()
 
@@ -64,6 +71,29 @@ args = parser.parse_args()
 # keep_full_length_versions = False
 
 
+def clear_folder(out_folder, subfolders=('originals', 'speedup')):
+    """Clear files in the given folder.
+
+    Args:
+        out_folder -- folder to clear files in
+        subfolders -- folders within out_folder to also clear
+
+    Folders in out_folder that are not included in subfolders will be ignored.
+    Subfolders will be cleared but not deleted.
+    If there are folders within the given subfolders, it may raise a PermissionError.
+    """
+    for subfolder in subfolders:
+        subfolder = Path(out_folder) / subfolder
+        if subfolder.is_dir():
+            for file in subfolder.iterdir():
+                # delete file
+                file.unlink()
+
+    for entry in Path(out_folder).iterdir():
+        if not entry.is_dir():
+            entry.unlink()
+
+
 def out_folder_empty(out_folder=args.out_folder, overwrite=args.clear_out_folder):
     """Check whether the output folder is empty, and clear it if applicable.
 
@@ -74,37 +104,18 @@ def out_folder_empty(out_folder=args.out_folder, overwrite=args.clear_out_folder
     delete all files in the folder and return True.
     """
     try:
-        out_folder_contents = os.listdir(out_folder)
+        out_folder_contents = Path(out_folder).iterdir()
     except FileNotFoundError:
         return True
     else:
         if overwrite:
             print(f"Clearing downloads folder '{out_folder}'..")
-            for file in out_folder_contents:
-                os.remove(os.path.join(out_folder, file))
+            clear_folder(out_folder)
             return True
         else:
             return not out_folder_contents
 
 
-def remove_duplicates(videos):
-    """Remove duplicate items in a list and return it.
-
-    Args:
-        videos -- input list
-    Returns the list with duplicate items removed, which may be identical to the original list if there were
-    no duplicate items.
-    """
-    seen = set()
-    unique_videos = []
-    for video in videos:
-        if video not in seen:
-            unique_videos.append(video)
-            seen.add(video)
-
-    return unique_videos
-
-
 def vods_list_from_file(path=args.vods_list_file_path):
     """Get a list of video URL's from a file.
 
@@ -119,13 +130,13 @@ def vods_list_from_file(path=args.vods_list_file_path):
             vods_list = vods_list_file.read().splitlines()
     except FileNotFoundError:
         print(f"List of VODs to download '{path}' not found.")
-        sys.exit()
+        sys.exit(2)
     else:
-        vods_list = remove_duplicates(vods_list)
+        vods_list = list(set(vods_list))
         return vods_list
 
 
-def download(vods_list, out_folder=args.out_folder, prefer_best_quality=args.prefer_best_quality):
+def download_and_speed_up(vods_list, out_folder=args.out_folder, prefer_best_quality=args.prefer_best_quality):
     """Download videos from a list of URL's using youtube-dl, and speed them up using ffmpeg.
 
     Downloads videos then automatically invokes the speed_up function on their filepath.
@@ -136,7 +147,8 @@ def download(vods_list, out_folder=args.out_folder, prefer_best_quality=args.pre
         vods_list -- a list of youtube-dl compatible video URL's. May contain only one element, but must be a list
     """
     ydl_args = {
-        'outtmpl': f'/{out_folder}/{YOUTUBE_DL_DEFAULT_OUTTMPL}',
+        # 'quiet': not args.verbose,
+        'outtmpl': f'/{out_folder}/originals/{YOUTUBE_DL_DEFAULT_OUTTMPL}',
         'progress_hooks': [speed_up],
     }
     if not prefer_best_quality:
@@ -163,19 +175,30 @@ def speed_up(video_download, speed=args.speed):
     """
     if not video_download['status'] == 'finished':
         return
-    filename, file_extension = os.path.splitext(video_download['filename'])
-    filename_no_extension = video_download['filename'][:-len(file_extension)]
+    # filename, file_extension = os.path.splitext(video_download['filename'])
+    # filename_no_extension = video_download['filename'][:-len(file_extension)]
+    filename = Path(video_download['filename'])
+    out_name = filename.with_stem(filename.stem + f'-{speed}x').name
+    out_filename = filename.parents[1] / 'speedup' / out_name
 
-    stream = ffmpeg.input(video_download['filename'])
+    stream = ffmpeg.input(str(filename))
     stream = ffmpeg.setpts(stream, f'(1/{speed})*PTS')
-    stream = ffmpeg.output(stream, f"{filename_no_extension}-{speed}x{file_extension}")
+    stream = ffmpeg.output(stream, str(out_filename))
+    # if not args.verbose:
+    #     stream = stream.global_args('-hide_banner')
+    #     # stream = stream.global_args('-loglevel', 'warning')
     ffmpeg.run(stream)
 
-    os.remove(video_download['filename'])
+    if not args.keep_original_parts:
+        filename.unlink()
 
 
-def combine_videos_in(folder=args.out_folder,
-                      out_filename=args.output_timelapse_filename, keep_parts=args.keep_timelapse_parts):
+def combine_videos_in(
+        folder=args.out_folder,
+        subfolder='/speedup/',
+        out_filename=args.output_timelapse_filename,
+        keep_parts=args.keep_timelapse_parts
+):
     """Combine videos in a folder into a single video using the ffmpeg concatenation demuxer.
 
     Original videos will be removed if keep_timelapse_parts is False.
@@ -183,40 +206,47 @@ def combine_videos_in(folder=args.out_folder,
     Args:
         folder -- the folder to combine videos in
     """
-    videos = os.listdir(folder)
-    parts_file_path = os.path.join(folder, '_parts.txt')
+    folder_path = Path(folder)
+    subfolder_path = folder_path / subfolder
+    videos = subfolder_path.iterdir()
+    parts_file_path = folder_path / '_parts.txt'
 
     with open(parts_file_path, 'w', encoding='utf-8') as parts_file:
-        parts_file.writelines([f"file '{os.path.join(folder, video)}'\n" for video in videos])
+        parts_file.writelines([f"file '{folder_path.joinpath(video)}'\n" for video in videos])
 
     stream = ffmpeg.input(parts_file_path, format='concat', safe=0)
-    stream = ffmpeg.output(stream, os.path.join(folder, out_filename), c='copy')
+    stream = ffmpeg.output(stream, folder_path.joinpath(out_filename), c='copy')
+    # if not args.verbose:
+    #     stream = stream.global_args('-hide_banner')
+    #     # stream = stream.global_args('-loglevel', 'warning')
     ffmpeg.run(stream)
 
     if not keep_parts:
-        os.remove(parts_file_path)
+        parts_file_path.unlink()
         for video in videos:
-            os.remove(os.path.join(folder, video))
+            folder_path.joinpath(video).unlink()
 
 
 def main():
     """Execute the script.
 
     First calls out_folder_empty and exits if result is False after printing an error message.
-    Then uses threading to download videos got from the vod_list_from_file function, using the download function.
+    Then uses threading to download videos got from the vod_list_from_file function, using the download_and_speed_up
+    function.
     Finally combines the downloaded videos using the combine_videos_in function.
     """
-    if not out_folder_empty():
-        print(f"The output folder '{args.out_folder}' contains files, please clear it or change the output folder.")
-        sys.exit()
+    if not args.no_clear:
+        if not out_folder_empty():
+            print(f"The output folder '{args.out_folder}' contains files, please clear it or change the output folder.")
+            sys.exit(1)
 
-    if not args.cmd_vods_list:
+    if not args.video_urls:
         vods_list = vods_list_from_file()
     else:
-        vods_list = args.cmd_vods_list
+        vods_list = args.video_urls
     print('Downloading and speeding up videos now with multithreading. You may see strange overlapping outputs.')
     with concurrent.futures.ThreadPoolExecutor() as threads:
-        threads.map(download, [[vod] for vod in vods_list])
+        threads.map(download_and_speed_up, [[vod] for vod in vods_list])
     print('Downloading finished, combining videos.')
     combine_videos_in(args.out_folder)
 
